###############################################################################################
#### - This script uses numpy, opencv, and shutil                                          ####
#### - https://pypi.org/project/opencv-python/                                             ####
#### - https://pypi.org/project/pytest-shutil/                                             ####
#### - https://pypi.org/project/numpy/                                                     ####
#### - Just set the top two variables ("setMipLevel" and "fileInDir" and execute           ####
#### - All sequences need to live in a folder named after the resolution of the images     ####
#### - Example - D:/Perforce/EXR_Sequences/Smoke/2048x2048/                                ####
####             D:/Perforce/EXR_Sequences/Smoke/1024x1024/                                ####
####             D:/Perforce/EXR_Sequences/Smoke/512x512/                                  ####
#### - For GPU optimized streaming into Unreal all Exr Sequences should be uncompressed    ####
#### - A mix of uncompressed and uncompressed in the mip levels will break the chain       ####
###############################################################################################

import cv2 as cv
import numpy as np
import glob
import os
import shutil

setMipLevel = 3
fileInDir = glob.glob("C:\\Users\\User\\Desktop\\smokeCards\\*.exr")

#### Gets file path to parent sequence ####
grabFirst = fileInDir[0]
splitFile = grabFirst.split('\\')[-1]
getParentPath = grabFirst.replace(splitFile,'')

#### Getting image resolution ####
img = cv.imread(grabFirst, cv.IMREAD_UNCHANGED)
height = np.size(img, 0)
width = np.size(img, 1)

#### Creates folders ####
def createFolders(dirName):
    isThere = os.path.isdir(dirName)
    if isThere == False:
        os.makedirs(dirName)

#### Builds folder path for LOD0 and copies sequence to LOD0 folder ####
LOD0_folderName = getParentPath + str(width) + 'x' + str(height)
createFolders(LOD0_folderName)

for x in fileInDir:
    getFileName = x.split('\\')[-1]
    newFile = LOD0_folderName + '\\' + getFileName
    if os.path.isfile(newFile) == 0:
        shutil.copyfile(x, newFile)
        print('copying ' + newFile + ' to correct file path')

#### Creates mipped EXR files ####
def createFiles(file, mipWidth, mipHeight, folderPath):
    getName = file.split('\\')[-1]
    imageSize = (int(mipWidth), int(mipHeight))
    readFile = cv.imread(file, cv.IMREAD_UNCHANGED)
    resizeFile = cv.resize(readFile, imageSize, interpolation = cv.INTER_LANCZOS4)
    newFile = folderPath + '\\' + getName
    cv.imwrite(newFile, resizeFile, [cv.IMWRITE_EXR_TYPE, cv.IMWRITE_EXR_TYPE_HALF])
    print("saving mipped file to " + newFile)

#### Does math for the mip levels, creates mipped folders and files ####
for index in range(setMipLevel):
    if index == 0:
        newWidth = width / 2
        newHeight = height / 2
        LOD1FolderPath = getParentPath + str(int(newWidth)) + 'x' + str(int(newHeight))
        createFolders(LOD1FolderPath)
        for file in fileInDir:
            createFiles(file, newWidth, newHeight, LOD1FolderPath)
    else:
        mipWidth = newWidth / 2
        mipHeight = newHeight / 2
        newWidth = mipWidth
        newHeight = mipHeight
        lowerMipFolderPath = getParentPath + str(int(newWidth)) + 'x' + str(int(newHeight))
        createFolders(lowerMipFolderPath)
        for file in fileInDir:
            createFiles(file, mipWidth, mipHeight, lowerMipFolderPath)